 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: VueDateEditorWidget
  * Author:   Louis
  * Date:     2021/11/5 16:32
  */
 package com.eco.plugin.zgtpbxdate.widget;


 import com.eco.plugin.zgtpbxdate.editors.DateTypeEditor;
 import com.eco.plugin.zgtpbxdate.editors.ReturnTypeEditor;
 import com.eco.plugin.zgtpbxdate.items.FirstDayOfWeekItems;
 import com.fr.base.BaseFormula;
 import com.fr.base.ParameterMapNameSpace;
 import com.fr.base.Utils;
 import com.fr.data.core.FormatField;
 import com.fr.design.designer.properties.items.Item;
 import com.fr.form.FormUtils;
 import com.fr.form.ui.DataControl;
 import com.fr.form.ui.DirectWriteEditor;
 import com.fr.form.ui.Widget;
 import com.fr.form.ui.WidgetValue;
 import com.fr.form.ui.concept.data.ValueInitializer;
 import com.fr.general.ComparatorUtils;
 import com.fr.general.DateUtils;
 import com.fr.json.JSONArray;
 import com.fr.json.JSONException;
 import com.fr.json.JSONObject;
 import com.fr.script.Calculator;
 import com.fr.stable.ArrayUtils;
 import com.fr.stable.CommonUtils;
 import com.fr.stable.StringUtils;
 import com.fr.stable.UtilEvalError;
 import com.fr.stable.core.NodeVisitor;
 import com.fr.stable.script.CalculatorProvider;
 import com.fr.stable.web.Repository;
 import com.fr.stable.web.SessionProvider;
 import com.fr.stable.xml.XMLPrintWriter;
 import com.fr.stable.xml.XMLableReader;
 import com.fr.web.utils.WebUtils;

 import javax.servlet.http.HttpServletRequest;
 import java.text.SimpleDateFormat;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.Set;

 import static com.eco.plugin.zgtpbxdate.creator.XVueDateEditor.*;

 /**
  * <Function Description><br>
  * <VueDateEditor>
  *
  * @author wink
  * @since 1.0.0
  */
 public class VueDateEditor extends DirectWriteEditor implements DataControl {
     private static final long serialVersionUID = 1406480833692359017L;
     public static String oldDateTimeWidgetTag = "com.fr.report.web.ui.DateTimeEditor";
     public static String oldDateTimeFormat = "yyyy-MM-dd HH:mm";
     public static final String SHOW_DATE_FM = "showDateFM";
     public static final String RETURN_FORMAT_FM = "returnFormatFM";
     private String startDate;
     private String endDate;
     private BaseFormula startDateFM;
     private BaseFormula endDateFM;
     //     private String formatText = "yyyy-MM-dd";
//     private String formatText = "yyyy-MM-dd hh:mm:ss";
     private ValueInitializer widgetValue = new WidgetValue();
     // 日期类型和格式
     private String dateFormat = DateTypeEditor.DATE;
     // 显示日期格式
     private String showDate;
     private BaseFormula showDateFM;
     // 返回格式
     private String returnFormat;
     private BaseFormula returnFormatFM;
     // 周起始日
     private int firstDayOfWeek = FirstDayOfWeekItems.SUNDAY;
     // 返回类型
     private String returnType = ReturnTypeEditor.STRING;
     // 数组拼接符
     private String dateSeparator = ",";
     // 配置项
     private String configurationItem = "";

     public VueDateEditor() {
     }

     @Override
     public String getXType() {
         return "custom_datetime";
     }

     @Override
     public String getFormatText() {
         return this.showDate;
     }

     public void setFormatText(String showDate) {
         this.showDate = showDate;
     }

     @Override
     public int[] getValueType() {
         return new int[]{TYPE_DATE, TYPE_FORMULA, TYPE_DATABINDING};
     }

     @Override
     public ValueInitializer getWidgetValue() {
         return this.widgetValue;
     }

     @Override
     public void setWidgetValue(ValueInitializer valueInitializer) {
         this.widgetValue = valueInitializer;
     }

     @Override
     public String[] dependence(CalculatorProvider calculator) {
         Set<String> hashSet = new HashSet<>();
         if (this.widgetValue != null) {
             Collections.addAll(hashSet, this.widgetValue.dependence(calculator));
         }
         Collections.addAll(hashSet, FormUtils.getDependence(this.getStartDate(), calculator));
         Collections.addAll(hashSet, FormUtils.getDependence(this.getEndDate(), calculator));
         return hashSet.toArray(new String[0]);
     }

     public Object getStartDateValue(Calculator calculator) throws Exception {
         if (this.startDateFM != null) {
             return calculator.eval(this.startDateFM);
         } else {
             return this.startDate != null ? DateUtils.string2Date(this.startDate, true) : null;
         }
     }

     public Object getEndDateValue(Calculator calculator) throws Exception {
         if (this.endDateFM != null) {
             return calculator.eval(this.endDateFM);
         } else {
             return this.endDate != null ? DateUtils.string2Date(this.endDate, true) : null;
         }
     }

     public Object getStartDate() {
         if (this.startDateFM != null) {
             return this.startDateFM;
         } else {
             return this.startDate != null ? DateUtils.string2Date(this.startDate, true) : null;
         }
     }

     public void setStartDate(Object startDate) {
         if (startDate instanceof BaseFormula) {
             this.startDateFM = (BaseFormula) startDate;
             this.startDate = null;
         } else if (startDate instanceof Date) {
             this.startDateFM = null;
             this.startDate = DateUtils.getDate2Str("MM/dd/yyyy", (Date) startDate);
         } else {
             this.startDateFM = null;
             this.startDate = null;
         }

     }

     public Object getEndDate() {
         if (this.endDateFM != null) {
             return this.endDateFM;
         } else {
             return this.endDate != null ? DateUtils.string2Date(this.endDate, true) : null;
         }
     }

     public void setEndDate(Object endDate) {
         if (endDate instanceof BaseFormula) {
             this.endDateFM = (BaseFormula) endDate;
             this.endDate = null;
         } else if (endDate instanceof Date) {
             this.endDateFM = null;
             this.endDate = DateUtils.getDate2Str("MM/dd/yyyy", (Date) endDate);
         } else {
             this.endDateFM = null;
             this.endDate = null;
         }

     }

     public String getStartText() {
         return this.startDate;
     }

     public void setStartText(String startText) {
         this.startDate = startText;
     }

     public String getEndText() {
         return this.endDate;
     }

     public void setEndText(String endText) {
         this.endDate = endText;
     }

     public BaseFormula getStartDateFM() {
         return this.startDateFM;
     }

     public void setStartDateFM(BaseFormula formula) {
         this.startDateFM = formula;
     }

     public BaseFormula getEndDateFM() {
         return this.endDateFM;
     }

     public void setEndDateFM(BaseFormula formula) {
         this.endDateFM = formula;
     }

     @Override
     public void createValueResult(CalculatorProvider calculator, JSONObject widgetResult) {
         if (this.widgetValue != null) {
             calculator.setAttribute(Widget.NAME, this.getWidgetName().toUpperCase());
             Object result = this.getWidgetValue().executeResult(calculator);
             if (StringUtils.equals(this.returnType, ReturnTypeEditor.STRING)
                     && result instanceof Date && this.showDate != null) {
                 result = (new SimpleDateFormat(this.showDate)).format((Date) result);
             }
             widgetResult.put(this.widgetName.toUpperCase(), result == null ? "" : result);
         }
     }

     @Override
     public String getDataBindDefaultValue(CalculatorProvider calculator) {
         if (this.widgetValue == null) {
             this.setWidgetValue(new WidgetValue());
         }

         Object result = this.getWidgetValue().executeResult(calculator);
         if (result == null) {
             return null;
         } else {
             return result instanceof Date && this.showDate != null ? (new SimpleDateFormat(this.showDate)).format((Date) result) : Utils.objectToString(result);
         }
     }

     @Override
     public JSONObject createJSONConfig(Repository repository, Calculator calculator, NodeVisitor nodeVisitor) throws JSONException {
         JSONObject config = super.createJSONConfig(repository, calculator, nodeVisitor);
         config.put("allowBlank", this.allowBlank);
         Object value = config.get("value");
         if (value == null) {
             config.remove("value");
         } else if (value instanceof Date) {
             config.put("value", (new JSONObject()).put("date_milliseconds", ((Date) value).getTime()));
         }

         if (StringUtils.isNotEmpty(this.showDate)) {
             config.put("format", this.showDate);
             if (!ArrayUtils.contains(FormatField.getInstance().getDateFormatArray(), this.showDate)) {
                 config.put("customFormat", true);
             }
         }
         try {
             if (StringUtils.isNotEmpty(this.startDate)) {
                 config.put("startDate", this.value2Config(this.startDate, calculator));
             } else if (this.startDateFM != null) {
                 config.put("startDate", this.value2Config(calculator.evalValue(this.startDateFM.getContent()), calculator));
             }

             if (StringUtils.isNotEmpty(this.endDate)) {
                 config.put("endDate", this.value2Config(this.endDate, calculator));
             } else if (this.endDateFM != null) {
                 config.put("endDate", this.value2Config(calculator.evalValue(this.endDateFM.getContent()), calculator));
             }

             if (StringUtils.isNotBlank(this.getDateFormat())) {
                 config.put(DATE_FORMAT, this.getDateFormat());
             }
             if (StringUtils.isNotBlank(this.getShowDate())) {
                 config.put(SHOW_DATE, this.getShowDate());
             }
             if (!CommonUtils.isNull(this.getShowDateFM())) {
                 config.put(SHOW_DATE, calculator.evalValue(this.getShowDateFM()));
             }
             if (StringUtils.isNotBlank(this.getReturnFormat())) {
                 config.put(RETURN_FORMAT, this.getReturnFormat());
             }
             if (!CommonUtils.isNull(this.getReturnFormatFM())) {
                 config.put(RETURN_FORMAT, calculator.evalValue(this.getReturnFormatFM()));
             }
         } catch (UtilEvalError e) {
//             LogKit.error(e.getMessage(), e);
         }
         config.put(FIRST_DAY_OF_WEEK, this.getFirstDayOfWeek());

         if (StringUtils.isNotBlank(this.getReturnType())) {
             config.put(RETURN_TYPE, this.getReturnType());
         }

         if (StringUtils.isNotBlank(this.getDateSeparator())) {
             config.put(DATE_SEPARATOR, this.getDateSeparator());
         }

         if (StringUtils.isNotBlank(this.getConfigurationItem())) {
             config.put(CONFIGURATION_ITEM, this.getConfigurationItem());
         }

         String[] dependence = this.dependence(calculator);
         if (!ArrayUtils.isEmpty(dependence)) {
             config.put("dependence", dependence);
         }
         return config;
     }

     @Override
     public JSONArray createJSONData(SessionProvider sessionProvider, Calculator calculator, HttpServletRequest request) throws Exception {
         JSONArray data = super.createJSONData(sessionProvider, calculator, request);
         ParameterMapNameSpace parameterMapNameSpace = ParameterMapNameSpace.create(sessionProvider.getOriginalParameterMap());
         DependenceNameSpace dependenceNameSpace = new DependenceNameSpace(WebUtils.getHTTPRequestParameter(request, "dependence"));
         calculator.pushNameSpace(parameterMapNameSpace);
         calculator.pushNameSpace(dependenceNameSpace);
         JSONObject dateJson = JSONObject.create();
         dateJson.put("startDate", this.value2Config(this.getStartDateValue(calculator), calculator));
         dateJson.put("endDate", this.value2Config(this.getEndDateValue(calculator), calculator));
         data.put(dateJson);
         calculator.removeNameSpace(parameterMapNameSpace);
         calculator.removeNameSpace(dependenceNameSpace);
         return data;
     }

     @Override
     public Object value2Config(Object value, CalculatorProvider calculator) {
         String config;
         if (value instanceof Date) {
             config = (new SimpleDateFormat(this.showDate)).format((Date) value);
         } else if (value == null) {
             config = "";
         } else {
             String str = Utils.objectToString(value);
             try {
                 Date var5 = DateUtils.string2Date(str, false);
                 config = (new SimpleDateFormat(this.showDate)).format(var5);
             } catch (Exception e) {
                 config = value.toString();
             }
         }
         return config;
     }

     @Override
     public String[] supportedEvents() {
         return new String[]{"afterinit", "afteredit", "stopedit"};
     }

     @Override
     public boolean equals(Object obj) {
         return obj instanceof VueDateEditor && super.equals(obj)
                 && ComparatorUtils.equals(this.startDate, ((VueDateEditor) obj).startDate)
                 && ComparatorUtils.equals(this.endDate, ((VueDateEditor) obj).endDate)
                 && StringUtils.equals(this.returnType, ((VueDateEditor) obj).returnType);
     }

     @Override
     public void readXML(XMLableReader reader) {
         super.readXML(reader);
         if (reader.isChildNode()) {
             String tagName = reader.getTagName();
             if ("DateAttr".equals(tagName)) {
                 String element;
                 if ((element = reader.getAttrAsString("format", null)) != null) {
                     this.setFormatText(element);
                 }
                 if ((element = reader.getAttrAsString("start", null)) != null) {
                     this.startDate = element;
                 } else if ((element = reader.getAttrAsString("startdatefm", null)) != null) {
                     this.startDateFM = BaseFormula.createFormulaBuilder().build(element);
                 }
                 if ((element = reader.getAttrAsString("end", null)) != null) {
                     this.endDate = element;
                 } else if ((element = reader.getAttrAsString("enddatefm", null)) != null) {
                     this.endDateFM = BaseFormula.createFormulaBuilder().build(element);
                 }
                 this.setShowDate(reader.getAttrAsString(SHOW_DATE, ""));
                 this.setShowDateFM(BaseFormula.createFormulaBuilder().build(reader.getAttrAsString(SHOW_DATE_FM, "")));
                 this.setReturnFormat(reader.getAttrAsString(RETURN_FORMAT, ""));
                 this.setReturnFormatFM(BaseFormula.createFormulaBuilder().build(reader.getAttrAsString(RETURN_FORMAT_FM, "")));
                 this.setFirstDayOfWeek(reader.getAttrAsInt(FIRST_DAY_OF_WEEK, FirstDayOfWeekItems.SUNDAY));
                 this.setReturnType(reader.getAttrAsString(RETURN_TYPE, ReturnTypeEditor.STRING));
                 this.setDateFormat(reader.getAttrAsString(DATE_FORMAT, DateTypeEditor.DATE));
                 this.setDateSeparator(reader.getAttrAsString(DATE_SEPARATOR, ""));
                 this.setConfigurationItem(reader.getAttrAsString(CONFIGURATION_ITEM, ""));
             } else if ("widgetValue".equals(tagName)) {
                 this.widgetValue = new WidgetValue();
                 reader.readXMLObject(this.widgetValue);
             }
         }
         if (ComparatorUtils.equals(reader.getAttrAsString("class", null), oldDateTimeWidgetTag) && reader.getAttrAsString("format", (String) null) == null) {
             this.setFormatText(oldDateTimeFormat);
         }
     }

     @Override
     public void writeXML(XMLPrintWriter writer) {
         super.writeXML(writer);
         writer.startTAG("DateAttr");
         if (!ComparatorUtils.equals(this.getFormatText(), this.getDefaultFormatText())) {
             writer.attr("format", this.getFormatText());
         }

         if (StringUtils.isNotEmpty(this.startDate)) {
             writer.attr("start", this.startDate);
         } else if (this.startDateFM != null) {
             writer.attr("startdatefm", this.startDateFM.getContent());
         }

         if (StringUtils.isNotEmpty(this.endDate)) {
             writer.attr("end", this.endDate);
         } else if (this.endDateFM != null) {
             writer.attr("enddatefm", this.endDateFM.getContent());
         }

         if (StringUtils.isNotBlank(this.getDateFormat())) {
             writer.attr(DATE_FORMAT, this.getDateFormat());
         }
         if (StringUtils.isNotBlank(this.getShowDate())) {
             writer.attr(SHOW_DATE, this.getShowDate());
         }
         if (this.getShowDateFM() != null && StringUtils.isNotBlank(this.getShowDateFM().getContent())) {
             writer.attr(SHOW_DATE_FM, this.getShowDateFM().getContent());
         }
         if (StringUtils.isNotBlank(this.getReturnFormat())) {
             writer.attr(RETURN_FORMAT, this.getReturnFormat());
         }
         if (this.getReturnFormatFM() != null && StringUtils.isNotBlank(this.getReturnFormatFM().getContent())) {
             writer.attr(RETURN_FORMAT_FM, this.getReturnFormatFM().getContent());
         }
         writer.attr(FIRST_DAY_OF_WEEK, this.getFirstDayOfWeek());
         if (StringUtils.isNotBlank(this.getReturnType())) {
             writer.attr(RETURN_TYPE, this.getReturnType());
         }
         if (StringUtils.isNotBlank(this.getDateSeparator())) {
             writer.attr(DATE_SEPARATOR, this.getDateSeparator());
         }
         if (StringUtils.isNotBlank(this.getConfigurationItem())) {
             writer.attr(CONFIGURATION_ITEM, this.getConfigurationItem());
         }

         writer.end();
         if (this.widgetValue != null) {
             this.widgetValue.writeXML(writer);
         }
     }

     protected String getDefaultFormatText() {
         return "yyyy-MM-dd";
     }

     public String getShowDate() {
         return showDate;
     }

     public void setShowDate(String showDate) {
         this.showDate = showDate;
     }

     public BaseFormula getShowDateFM() {
         return showDateFM;
     }

     public void setShowDateFM(BaseFormula showDateFM) {
         this.showDateFM = showDateFM;
     }

     public int getFirstDayOfWeek() {
         return firstDayOfWeek;
     }

     public void setFirstDayOfWeek(Object firstDayOfWeek) {
         Item selectedItem = (Item) firstDayOfWeek;
         this.firstDayOfWeek = Integer.parseInt(String.valueOf(selectedItem.getValue()));
     }

     public void setFirstDayOfWeek(int firstDayOfWeek) {
         this.firstDayOfWeek = firstDayOfWeek;
     }

     public String getConfigurationItem() {
         return configurationItem;
     }

     public void setConfigurationItem(String configurationItem) {
         this.configurationItem = configurationItem;
     }

     public String getDateFormat() {
         return dateFormat;
     }

     public void setDateFormat(String dateFormat) {
         this.dateFormat = dateFormat;
     }

     public String getReturnFormat() {
         return returnFormat;
     }

     public void setReturnFormat(String returnFormat) {
         this.returnFormat = returnFormat;
     }

     public BaseFormula getReturnFormatFM() {
         return returnFormatFM;
     }

     public void setReturnFormatFM(BaseFormula returnFormatFM) {
         this.returnFormatFM = returnFormatFM;
     }

     public int getReturnTypeInt() {
         if (StringUtils.equals(this.returnType, ReturnTypeEditor.DATE)) {
             return 0;
         } else if (StringUtils.equals(this.returnType, ReturnTypeEditor.STRING)) {
             return 1;
         } else {
             return 1;
         }
     }

     public String getReturnType() {
         return returnType;
     }

     public void setReturnType(int returnType) {
         if (ComparatorUtils.equals(returnType, 0)) {
             this.returnType = ReturnTypeEditor.DATE;
         } else if (ComparatorUtils.equals(returnType, 1)) {
             this.returnType = ReturnTypeEditor.STRING;
         } else {
             this.returnType = ReturnTypeEditor.STRING;
         }
     }

     public void setReturnType(String returnType) {
         this.returnType = returnType;
     }

     public String getDateSeparator() {
         return dateSeparator;
     }

     public void setDateSeparator(String dateSeparator) {
         this.dateSeparator = dateSeparator;
     }
 }